# -*- coding: utf-8 -*-
"""
@Author: HuangJingCan
@Date: 2021-01-08 09:56:37
:LastEditTime: 2021-05-17 10:04:43
:LastEditors: HuangJingCan
@Description: 
"""
import hashlib
import copy
from seven_framework.base_model import *
from seven_dyapp.handlers.base.douyin_base import *
from seven_dyapp.handlers.base.seven_helper import *


class DouYinError(Exception):
    def __init__(self, msg):
        super(DouYinError, self).__init__(msg)


class DouYinPayRequest(object):
    """配置账号信息"""
    # =======【基本信息设置】=====================================
    # 抖音公众号身份的唯一标识。审核通过后，在抖音发送的邮件中查看
    APPID = ""
    # API密钥，需要在商户后台设置
    APIKEY = ""
    #开发者自定义字段，回调原样回传
    CPEXTRA = ""
    #商户自定义回调地址
    NOTIFYURL = ""
    #担保交易Token（令牌）
    TOKEN = ""
    #担保交易盐值
    SALT = ""
    # 日志
    logger_error = Logger.get_logger_by_name("log_error")

    def __init__(self):
        """
        :description: 初始化
        :last_editors: HuangJingCan
        """
        payconfig = config.get_value("douyin_pay")
        self.APPID = config.get_value("app_id")
        self.APIKEY = payconfig['api_key']
        self.CPEXTRA = payconfig['cp_extra']
        self.NOTIFYURL = payconfig['notify_url']
        self.TOKEN = payconfig['token']
        self.SALT = payconfig['salt']

    def create_order(self, total_amount=1, subject="", body="", valid_time=900, thirdparty_id="", disable_msg=None, msg_page="", store_uid=""):
        """
        :description: 服务端支付预下单
        :param total_amount:支付价格; 接口中参数支付金额单位为[分]|默认 1
        :param subject: 商品描述; 长度限制 128 字节，不超过 42 个汉字 |默认 自己约定一个比如 “福小宠商品”*|
        :param body:商品详情|默认 自己约定一个比如 “福小宠商品”
        :param valid_time:订单过期时间(秒); 最小 15 分钟，最大两天|默认 900
        :param thirdparty_id:服务商模式接入必传,第三方平台服务商 id，非服务商模式留空|默认传空字符串
        :param disable_msg:是否屏蔽担保支付的推送消息，1-屏蔽 0-非屏蔽，接入 POI 必传|默认传None
        :param msg_page:担保支付消息跳转页|默认传空字符串
        :param store_uid:多门店模式下必传,多门店模式下，门店 uid|默认传空字符串
        :last_editors: HuangJingCan
        """
        app_id = self.APPID
        #out_order_no 开发者服务端的唯一订单号
        out_order_no = SevenHelper.create_order_id()
        #回调地址
        notify_url = self.NOTIFYURL
        cp_extra = self.CPEXTRA
        # 在这里将订单信息存入数据库

        data = {}
        data['out_order_no'] = out_order_no
        data['cp_extra'] = cp_extra
        data['notify_url'] = notify_url
        data['total_amount'] = str(int(decimal.Decimal(str(total_amount)) * 100))
        data['subject'] = subject
        data['body'] = body
        data['valid_time'] = valid_time
        if disable_msg:
            data['disable_msg'] = int(disable_msg)
        if msg_page:
            data['msg_page'] = msg_page
        if store_uid:
            data['store_uid'] = store_uid

        sign = self.get_tt_sign(data)
        # 如果有第三方平台服务商id:thirdparty_id字段请把他放入data的字典里面
        data['sign'] = sign
        data['app_id'] = app_id

        if thirdparty_id:
            data['thirdparty_id'] = thirdparty_id

        redis_key = "douyin_order_id:" + str(out_order_no)
        order_id = DouYinHelper.redis_init().get(redis_key)
        if order_id:
            return order_id

        url = "https://developer.toutiao.com/api/apps/ecpay/v1/create_order"
        headers = {"Content-type": "application/json"}
        response = requests.post(url, headers=headers, data=json.dumps(data))
        if response.status_code == 200:
            resp = json.loads(response.text)
            # err_no	number	    状态码 0-业务处理成功
            # err_tips	string	    错误提示信息，常见错误处理可参考附录常见问题章节
            # data	    orderInfo	拉起收银台的 orderInfo
            # {"err_no": 2000, "err_tips": "单号记录不存在", "data": null}
            if resp['err_no'] == 0:
                order_id = str(resp['data']['order_id'])
                DouYinHelper.redis_init().set(redis_key, order_id, ex=3600 * 1)
                return resp
            else:
                self.logger_error.error(f"【{out_order_no},统一下单获取order_id】" + resp['err_tips'])
                return DouYinError(resp['err_tips'])
        else:
            self.logger_error.error(f"【{out_order_no},统一下单获取order_id】" + response.text)
            return DouYinError("支付接口报错，请重试")

    def query_order(self, out_order_no="", thirdparty_id=""):
        """
        :description: 查询订单
        :param out_order_no：开发者侧的订单号, 不可重复
        :param thirdparty_id: 服务商模式接入必传  第三方平台服务商 id，非服务商模式留空字符串|默认值 空字符串
        :last_editors: HuangJingCan
        """
        try:
            app_id = self.APPID
            data = {}
            data['out_order_no'] = out_order_no
            sign = self.get_tt_sign(data)
            data['sign'] = sign
            data['app_id'] = app_id
            if thirdparty_id:
                data['thirdparty_id'] = thirdparty_id

            url = "https://developer.toutiao.com/api/apps/ecpay/v1/query_order"
            headers = {"Content-type": "application/json"}
            response = requests.post(url, headers=headers, data=json.dumps(data))
            if response.status_code == 200:
                resp = json.loads(response.text)
                # {"err_no": 2000, "err_tips": "单号记录不存在", "out_order_no": "", "order_id": "", "payment_info": null}
                # payment_info 返回值结构如下：
                # {
                #     "total_fee": 1200,
                #     "order_status": "PROCESSING-处理中|SUCCESS-成功|FAIL-失败|TIMEOUT-超时",
                #     "pay_time": "支付时间",
                #     "way": 1,
                #     "channel_no": "渠道单号",
                #     "channel_gateway_no": "渠道网关号"
                # }
                if resp['err_no'] == 0:
                    return resp
                else:
                    self.logger_error.error(f"【{out_order_no},查询订单】" + resp['err_tips'])
                    return DouYinError(resp['err_tips'])
            else:
                self.logger_error.error(f"【查询订单】" + response.text)
                return DouYinError("查询订单出现异常，请重试")
        except Exception as ex:
            self.logger_error.error("【查询订单】" + str(ex))
            return DouYinError("查询订单出现异常，请重试")

    def create_refund(self, out_order_no="", refund_amount=1, reason="", thirdparty_id="", disable_msg=None, msg_page="", all_settle=None):
        """
        :description: 服务端退款请求
        :param out_order_no:商户分配订单号，标识进行退款的订单，开发者服务端的唯一订单号
        :param refund_amount: 退款金额，单位[分]|默认 1
        :param reason:退款理由，长度上限 100|默认 由开发者定 例如：“7天无理由退款”
        :param thirdparty_id:服务商模式接入必传,第三方平台服务商 id，非服务商模式留空|默认传空字符串
        :param disable_msg:是否屏蔽担保支付的推送消息，1-屏蔽 0-非屏蔽，接入 POI 必传|默认传None
        :param msg_page:担保支付消息跳转页|默认传空字符串
        :param all_settle:是否为分账后退款，1-分账后退款；0-分账前退款。分账后退款会扣减可提现金额，请保证余额充足*|默认传None
        :last_editors: HuangJingCan
        """
        app_id = self.APPID
        #out_order_no 开发者服务端的唯一订单号
        out_refund_no = SevenHelper.create_order_id()
        #回调地址
        notify_url = self.NOTIFYURL
        cp_extra = self.CPEXTRA
        # 在这里将订单信息存入数据库
        data = {}
        data['out_refund_no'] = out_refund_no
        #cp_extra 开发者自定义字段，回调原样回传
        data['cp_extra'] = cp_extra
        data['notify_url'] = notify_url
        data['out_order_no'] = out_order_no
        data['refund_amount'] = str(int(decimal.Decimal(str(refund_amount)) * 100))
        data['reason'] = reason
        if disable_msg:
            data['disable_msg'] = int(disable_msg)
        if msg_page:
            data['msg_page'] = msg_page
        if all_settle:
            data['all_settle'] = int(all_settle)

        sign = self.get_tt_sign(data)
        # 如果有第三方平台服务商 id :thirdparty_id字段请把他放入data.update的字典里面
        data['sign'] = sign
        data['app_id'] = app_id
        if thirdparty_id:
            data['thirdparty_id'] = thirdparty_id

        url = "https://developer.toutiao.com/api/apps/ecpay/v1/create_refund"
        headers = {"Content-type": "application/json"}
        response = requests.post(url, headers=headers, data=json.dumps(data))
        if response.status_code == 200:
            resp = json.loads(response.text)
            if resp['err_no'] == 0:
                return resp

    def query_refund(self, out_refund_no="", thirdparty_id=""):
        """
        :desciption:退款查询
        :param out_refund_no:开发者侧的退款单号, 不可重复
        :param thirdparty_id:服务商模式接入必传  第三方平台服务商 id，非服务商模式留空字符串
        :last_editors: HuangJingCan
        """
        app_id = self.APPID
        data = {}
        data['out_refund_no'] = out_refund_no
        sign = self.get_tt_sign(data)
        data['sign'] = sign
        data['app_id'] = app_id
        if thirdparty_id:
            data['thirdparty_id'] = thirdparty_id

        url = "https://developer.toutiao.com/api/apps/ecpay/v1/query_refund"
        headers = {"Content-type": "application/json"}
        response = requests.post(url, headers=headers, data=json.dumps(data))
        if response.status_code == 200:
            resp = json.loads(response.text)
            if resp['err_no'] == 0:
                return resp

    def create_settle(self, out_order_no="", settle_desc="", settle_params="", thirdparty_id=""):
        """
        :description:服务端结算请求
        :param out_order_no:商户分配订单号，标识进行退款的订单，开发者服务端的唯一订单号
        :param settle_desc:结算描述 默认 自己约定一个比如 “福小宠商品结算”
        :param settle_params:其他分账方信息，分账分配参数 SettleParameter 数组序列化后生成的 json 格式字符串
        :param thirdparty_id：服务商模式接入必传,第三方平台服务商 id，非服务商模式留空|默认传空字符串
        :last_editors: HuangJingCan
        """
        app_id = self.APPID
        #out_order_no 开发者服务端的唯一订单号
        out_settle_no = SevenHelper.create_order_id()
        #回调地址
        notify_url = self.NOTIFYURL
        cp_extra = self.CPEXTRA
        # 在这里将分账信息存入数据库
        data = {}
        data['out_settle_no'] = out_settle_no
        #cp_extra 开发者自定义字段，回调原样回传
        data['cp_extra'] = cp_extra
        data['notify_url'] = notify_url
        data['out_order_no'] = out_order_no
        data['settle_desc'] = settle_desc
        if settle_params:
            data['settle_params'] = settle_params
        sign = self.get_tt_sign(data)
        data['sign'] = sign
        data['app_id'] = app_id
        if thirdparty_id:
            data['thirdparty_id'] = thirdparty_id

        url = "https://developer.toutiao.com/api/apps/ecpay/v1/settle"
        headers = {"Content-type": "application/json"}
        response = requests.post(url, headers=headers, data=json.dumps(data))
        if response.status_code == 200:
            resp = json.loads(response.text)
            if resp['err_no'] == 0:
                return resp

    def query_settle(self, out_settle_no="", thirdparty_id=""):
        """
        :desciption:结算查询
        :param out_settle_no:开发者侧的分账号, 不可重复
        :param thirdparty_id:服务商模式接入必传  第三方平台服务商 id，非服务商模式留空字符串
        :last_editors: HuangJingCan
        """
        app_id = self.APPID
        data = {}
        data['out_settle_no'] = out_settle_no
        sign = self.get_tt_sign(data)
        data['sign'] = sign
        data['app_id'] = app_id
        if thirdparty_id:
            data['thirdparty_id'] = thirdparty_id

        url = "https://developer.toutiao.com/api/apps/ecpay/v1/query_settle"
        headers = {"Content-type": "application/json"}
        response = requests.post(url, headers=headers, data=json.dumps(data))
        if response.status_code == 200:
            resp = json.loads(response.text)
            if resp['err_no'] == 0:
                return resp

    def call_back(self, data):
        """
        :description:支付/退款/结算回调
        :param params: 字典 由抖音端请求,通过params = json.loads(self.request.body)获得
        :last_editors: HuangJingCan
        """
        # data = {
        #     'timestamp': '1620464599',
        #     'nonce': '6115',
        #     'msg':
        #     '{"appid":"tt3a194ca4a5233849","cp_orderno":"2021050816471823375231","cp_extra":"hello world","way":"2","channel_no":"2021050822001459941406250991","channel_gateway_no":"12105080123729621931","payment_order_no":"2021050822001459941406250991","out_channel_order_no":"","total_amount":1}',
        #     'msg_signature': '69cbf84dff685cc43df65e288bf1c2607bc318ab',
        #     'type': 'payment'
        # }

        if data:
            #计算服务端sign
            service_signature = self.get_tt_callback_sign(data)
            #验证成功,进行业务处理
            log_msg = f'客户端签名:{data["msg_signature"]},服务端签名:{service_signature}'
            # logger_info.info(log_msg)
            if service_signature == data["msg_signature"]:
                #根据支付/退款/分账等回调json内容 在这边写业务逻辑
                #检测type字段判断是支付回调还是退款回调
                # type:payment 支付成功回调,refund退款成功回调,settle分账成功回调
                return  #to do

    def get_tt_sign(self, params):
        """
        :description: 获取头条请求sign
        :param params: 参数字典
        :return: sign
        :last_editors: HuangJingCan
        """
        params_copy = copy.deepcopy(params)
        for key, value in params_copy.items():
            if isinstance(value, int):
                params_copy[key] = str(value)
        params_copy['salt'] = self.SALT
        # params_copy['salt'] = 'your_payment_salt'
        params_list = sorted(list(params_copy.items()), key=lambda x: x[1])
        params_str = ''.join(f"{v}&" for k, v in params_list)[:-1]
        sign = hashlib.md5(params_str.encode('utf-8')).hexdigest()
        return sign

    def get_tt_callback_sign(self, params):
        """
        :description: 获取头条回调sign
        :param params: 参数字典
        :return: sign
        :last_editors: HuangJingCan
        """
        keys = ['type', 'msg_signature']
        params_copy = {key: params[key] for key in params if key not in keys}
        for key, value in params_copy.items():
            if isinstance(value, int):
                params_copy[key] = str(value)
        params_copy['token'] = self.TOKEN
        # params_copy['salt'] = 'your_payment_salt'
        params_list = sorted(list(params_copy.items()), key=lambda x: x[1])
        params_str = ''.join(f"{v}" for k, v in params_list)
        sign = CryptoHelper().sha1_encrypt(params_str)
        return sign